VERSION 5.00
Begin VB.Form frmMain 
   BackColor       =   &H00000000&
   Caption         =   "VB RenderToSurface Text Demo"
   ClientHeight    =   6765
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   7425
   BeginProperty Font 
      Name            =   "Tahoma"
      Size            =   14.25
      Charset         =   0
      Weight          =   400
      Underline       =   0   'False
      Italic          =   0   'False
      Strikethrough   =   0   'False
   EndProperty
   Icon            =   "frmMain.frx":0000
   LinkTopic       =   "Form1"
   ScaleHeight     =   6765
   ScaleWidth      =   7425
   StartUpPosition =   3  'Windows Default
   Begin VB.Timer Timer1 
      Interval        =   40
      Left            =   360
      Top             =   240
   End
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'  Application Title:       Render To Surface Text
'  Developer:               Eric DeBrosse [MS MVP]
'                           The DirectX Cool Fool!
'                           http://www.blown.com/dx/
'
'  File:        frmMain.frm
'  Content:     Draws text by rendering a character string to
'               a surface, then mapping the created surface onto
'               a quad so it can be rendered in world space.
'               Typically D3DXFont can only be used to render
'               into the viewport in 2D. This sample shows you
'               how to draw text in 3D space.
'
' Content is not to be duplicated without permission.
' (c)2002 Eric DeBrosse
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'-----------------------------------------------------------------------------
' variables
'-----------------------------------------------------------------------------
Dim g_oDX As DirectX8
Dim g_oD3D As Direct3D8
Dim g_oD3DX As D3DX8
Dim g_oD3DDevice As Direct3DDevice8
Dim g_d3dpp As D3DPRESENT_PARAMETERS

'texture used to render text
Dim m_oTexture As Direct3DTexture8

'our custom vertex type
Const MY_VERTEX_TYPE = D3DFVF_XYZ Or D3DFVF_DIFFUSE Or D3DFVF_TEX1

'structure for quad data
Private Type MYVERTEX
    x As Single
    y As Single
    z  As Single
    Diffuse As Long
    tu As Single
    tv As Single
End Type

Dim m_Quad(4) As MYVERTEX   '0-3

'-----------------------------------------------------------------------------
' Name: InitDeviceObjects
' Desc: Creates a texture and renders some text to it.
'-----------------------------------------------------------------------------
Private Sub InitDeviceObjects()
    Dim oTempFont As IFont      'IFont requires OLE automation
    Dim oFont As D3DXFont
    Dim oSurface As Direct3DSurface8
    Dim oRTS As D3DXRenderToSurface
    Dim DispMode As D3DDISPLAYMODE
    Dim rtsViewport As D3DVIEWPORT8
    Dim matWorld As D3DMATRIX
    Dim matProj As D3DMATRIX
    Dim rc As RECT
    Dim sDrawText As String
    
    On Error GoTo ErrOut
    
    With g_oD3DDevice
        'set the render states that we need
        .SetRenderState D3DRS_LIGHTING, 0&
        .SetRenderState D3DRS_CULLMODE, D3DCULL_NONE
        .SetRenderState D3DRS_ALPHABLENDENABLE, 1&
        
        'turn on min filtering since our text is often smaller than the original size
        .SetTextureStageState 0&, D3DTSS_MINFILTER, D3DTEXF_LINEAR
        .SetTextureStageState 0&, D3DTSS_MAGFILTER, D3DTEXF_LINEAR
    
        'set the world matrix to identity
        D3DXMatrixIdentity matWorld
        .SetTransform D3DTS_WORLD, matWorld
        
        'set the projection matrix
        D3DXMatrixPerspectiveFovLH matProj, 0.785!, 1!, 1!, 200!
        .SetTransform D3DTS_PROJECTION, matProj
    End With
    
    'This constant is used to set the size of render surface.  We will
    'use 256, since this is usually the maximum texture size that can
    'be used on older 3D cards.  Normally you would check the device
    'caps to see if the requested texture size is available.  We are
    'keeping the texture square just to keep things easy.
    Const SURFACE_SIZE = 256&
    
    'get the current display format
    g_oD3DDevice.GetDisplayMode DispMode
    
    'create a texture to draw text on (one mip-level) using format of display
    Set m_oTexture = g_oD3DX.CreateTexture(g_oD3DDevice, SURFACE_SIZE, SURFACE_SIZE, _
            1&, 0&, DispMode.Format, D3DPOOL_MANAGED)
    
    'create the font object used to draw the text.
    '(frmMain must have a TrueType font selected.)
    'convert the form's font object to an IFont object so we can get its handle.
    Set oTempFont = frmMain.Font
    Set oFont = g_oD3DX.CreateFont(g_oD3DDevice, oTempFont.hFont)
    Set oTempFont = Nothing
    
    'create a render surface using current display format
    Set oRTS = g_oD3DX.CreateRenderToSurface(g_oD3DDevice, SURFACE_SIZE, SURFACE_SIZE, _
            DispMode.Format, 0&, D3DFMT_UNKNOWN)
    
    'set viewport parameters
    With rtsViewport
        .Height = SURFACE_SIZE
        .Width = SURFACE_SIZE
        .MaxZ = 1!
    End With
    
    'get top-level surface (we only created one mip-level) from our texture
    Set oSurface = m_oTexture.GetSurfaceLevel(0&)
    
    'prepare to render to this surface and clear to a gray color
    oRTS.BeginScene oSurface, rtsViewport
    g_oD3DDevice.Clear 0&, ByVal 0&, D3DCLEAR_TARGET, &HFFC0C0C0, 1!, 0&
    
    'set position to begin drawing text
    rc.Left = 10&
    rc.Top = 20&
    
    sDrawText = "The DirectX Cool Fool!" & vbCrLf & _
            "Eric DeBrosse [MS MVP]" & vbCrLf & vbCrLf & vbCrLf & _
            "Be sure to visit my web site" & vbCrLf & _
            "for more DirectX samples!" & vbCrLf & vbCrLf & _
            "http://www.blown.com/dx/"
    
    'render text onto surface
    With oFont
        .Begin
        .DrawTextW sDrawText, Len(sDrawText), rc, DT_CALCRECT, 0&
        .DrawTextW sDrawText, Len(sDrawText), rc, DT_LEFT, &HFF000077
        .End
    End With
    
    'done rendering to surface
    oRTS.EndScene
    
    'done drawing with the font object
    Set oFont = Nothing
    
    'create data for a quad (x, y, z, tu, tv, Diffuse)
    m_Quad(0) = MakeVertex(-0.3, 0.3, 0, 0, 0, &HFFFFFFFF)
    m_Quad(1) = MakeVertex(0.3, 0.3, 0, 1, 0, &HFFFF5555)
    m_Quad(2) = MakeVertex(-0.3, -0.3, 0, 0, 1, &HFF5555FF)
    m_Quad(3) = MakeVertex(0.3, -0.3, 0, 1, 1, &HFF55FF55)
Exit Sub

ErrOut:
    'uh-oh, something went wrong!
    Debug.Print "Failed InitDeviceObjects: " & Hex(Err.Number)
End Sub

'-----------------------------------------------------------------------------
' Name: Form_Load
' Desc:
'-----------------------------------------------------------------------------
Private Sub Form_Load()
    'force form to become visible before rendering
    Me.Show
    DoEvents
    
    'initialize Direct3D and Direct3D device
    If InitD3D(Me.hWnd) = False Then
        MsgBox "Unable to create Direct3D device.", vbCritical, App.Title
        End
    End If
    
    'initialize objects
    InitDeviceObjects
End Sub

'-----------------------------------------------------------------------------
' Name: Form_Unload
' Desc:
'-----------------------------------------------------------------------------
Private Sub Form_Unload(Cancel As Integer)
    Call Cleanup
    End
End Sub

'-----------------------------------------------------------------------------
' Name: InitD3D
' Desc: Used to initialize Direct3D using HAL.
'-----------------------------------------------------------------------------
Private Function InitD3D(ByVal hWnd As Long) As Boolean
    On Error GoTo ErrOut
    
    'create DirectX objects
    Set g_oDX = New DirectX8
    Set g_oD3DX = New D3DX8
    
    'create the D3D object
    Set g_oD3D = g_oDX.Direct3DCreate()
    If g_oD3D Is Nothing Then
        InitD3D = False
        Exit Function
    End If
    
    'get the current format of the desktop
    'NOTE: device creation will fail if the format of the desktop is 32 bits
    'and the device only has 16 bit D3D modes (i.e. old Voodoo cards)
    Dim DispMode As D3DDISPLAYMODE
    g_oD3D.GetAdapterDisplayMode D3DADAPTER_DEFAULT, DispMode
    
    'set up a generic structure used to create the D3DDevice
    With g_d3dpp
        .Windowed = 1
        .SwapEffect = D3DSWAPEFFECT_FLIP
        .BackBufferFormat = DispMode.Format
        .BackBufferCount = 1
        .EnableAutoDepthStencil = 0
        .AutoDepthStencilFormat = D3DFMT_UNKNOWN
    End With
    
    'create the D3DDevice
    Set g_oD3DDevice = g_oD3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, _
                                           D3DCREATE_SOFTWARE_VERTEXPROCESSING, g_d3dpp)
    
    If g_oD3DDevice Is Nothing Then
        InitD3D = False
        Exit Function
    End If
    
    'device was created
    InitD3D = True
Exit Function

ErrOut:
    'failed to create device
    InitD3D = False
End Function

'-----------------------------------------------------------------------------
' Name: Cleanup
' Desc: Releases all previously initialized objects.
'-----------------------------------------------------------------------------
Private Sub Cleanup()
    'destroy our managed objects
    Set m_oTexture = Nothing
    
    'destroy D3D stuff
    Set g_oD3DX = Nothing
    Set g_oD3DDevice = Nothing
    Set g_oD3D = Nothing
    Set g_oDX = Nothing
End Sub

'-----------------------------------------------------------------------------
' Name: Render
' Desc: Draws the scene.
'-----------------------------------------------------------------------------
Private Sub Render()
    On Error Resume Next
    
    If g_oD3DDevice Is Nothing Then Exit Sub
    
    Select Case g_oD3DDevice.TestCooperativeLevel
        Case D3DERR_DEVICENOTRESET
            'try to reset device
            g_oD3DDevice.Reset g_d3dpp
            Exit Sub
            
        Case Is <> D3D_OK
            'dont bother rendering if we are not ready yet
            Exit Sub
    End Select
    
    On Error GoTo ErrOut
    
    'start the scene and clear the frame buffer
    g_oD3DDevice.BeginScene
    g_oD3DDevice.Clear 0&, ByVal 0&, D3DCLEAR_TARGET, 0&, 1!, 0&
    
    'set texture for quad
    g_oD3DDevice.SetTexture 0, m_oTexture
    
    'describe vertex data to the device
    g_oD3DDevice.SetVertexShader MY_VERTEX_TYPE
    
    'draw textured quad as a triangle strip
    g_oD3DDevice.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2&, m_Quad(0), Len(m_Quad(0))
    
    'end 3D scene
    g_oD3DDevice.EndScene
    
    'present the scene
    g_oD3DDevice.Present ByVal 0&, ByVal 0&, 0&, ByVal 0&
Exit Sub

ErrOut:
'try to render later
End Sub

'-----------------------------------------------------------------------------
' Name: Timer1_Timer
' Desc: A timer is an easy way to update our view.
'-----------------------------------------------------------------------------
Private Sub Timer1_Timer()
    Dim matView As D3DMATRIX
    Dim vEye As D3DVECTOR
    Dim vAt As D3DVECTOR
    Dim vUp As D3DVECTOR
    
    'update view
    vEye = Vec3(Cos(Timer / 3.145), 1, Sin(Timer / 3.145))
    vAt = Vec3(0, 0, 0)
    vUp = Vec3(0, 1, 0)
    
    D3DXMatrixLookAtLH matView, vEye, vAt, vUp
    g_oD3DDevice.SetTransform D3DTS_VIEW, matView
    
    'update scene
    Call Render
End Sub

'-----------------------------------------------------------------------------
' Name: MakeVertex
' Desc: Returns a filled MYVERTEX structure.
'-----------------------------------------------------------------------------
Private Function MakeVertex(ByVal x As Single, _
                            ByVal y As Single, _
                            ByVal z As Single, _
                            ByVal tu As Single, _
                            ByVal tv As Single, _
                            ByVal Diffuse As Long) As MYVERTEX
    With MakeVertex
        .x = x
        .y = y
        .z = z
        .tu = tu
        .tv = tv
        .Diffuse = Diffuse
    End With
End Function

'-----------------------------------------------------------------------------
' Name: Vec3
' Desc: Returns a filled D3DVECTOR structure.
'-----------------------------------------------------------------------------
Private Function Vec3(ByVal x As Single, _
                      ByVal y As Single, _
                      ByVal z As Single) As D3DVECTOR
    With Vec3
        .x = x
        .y = y
        .z = z
    End With
End Function

